/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ElmoAnonymousTaxon.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.elmo.impl;

import it.tidalwave.bluebill.taxonomy.TaxonomyConcept;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import javax.annotation.Nonnull;
import org.openrdf.elmo.annotations.rdf;
import org.openrdf.elmo.Entity;
import it.tidalwave.bluebill.taxonomy.AnonymousTaxon;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.elmo.ElmoTaxonomyVocabulary;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.logging.Logger;
import java.util.HashSet;
import org.openrdf.elmo.ResourceManager;
import org.openrdf.elmo.sesame.SesameManager;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ElmoAnonymousTaxon.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 **********************************************************************************************************************/
@rdf(ElmoTaxonomyVocabulary.URI_ANONYMOUS_TAXON)
public class ElmoAnonymousTaxon extends Support implements InternalElmoAnonymousTaxon
  {
    private static final String CLASS = ElmoAnonymousTaxon.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    @Nonnull
    private Repository repository;

    @Nonnull
    private Taxonomy taxonomy;

//    @inverseOf(ElmoTaxonomyVocabulary.URI_TAXONOMY_SYNONYM)
//    private Set<TaxonomyConcept> synonyms;

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public ElmoAnonymousTaxon (@Nonnull final Entity entity)
      {
        super(entity);
      }

    @Override
    public Object getSynonyms()
//    public Set<TaxonomyConcept> getSynonyms()
      {
        try // FIXME: workaround because the @rdf annotation of this.synonym doesn't work
          {
            final SesameManager em = (SesameManager)ElmoManagerThreadLocal.get();
            final ResourceManager<Resource> rm = em.getResourceManager();
            final RepositoryResult<Statement> triples = em.getConnection().getStatements(null,
                                                                                         new URIImpl(ElmoTaxonomyVocabulary.URI_TAXONOMY_SYNONYM),
                                                                                         rm.createResource(getLookup().lookup(Entity.class).getQName()));
            final Set<TaxonomyConcept> synonyms = new HashSet<TaxonomyConcept>();

            while (triples.hasNext())
              {
                final URI uri = (URI)triples.next().getSubject();
                synonyms.add(((InternalElmoTaxonomy)taxonomy)._findConcept(uri.stringValue()));
              }

            return synonyms;
          }
        catch (NotFoundException e)
          {
            throw new RuntimeException(e);
          }
        catch (RepositoryException e)
          {
            throw new RuntimeException(e);
          }
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void setBindings (@Nonnull final Repository repository,
                             @Nonnull final Taxonomy taxonomy)
      {
        logger.finest("setBindings(%s, %s) - @%x", repository, taxonomy, System.identityHashCode(this));

        this.taxonomy = taxonomy;
        this.repository = repository;
      }

    public Map<Locale, String> getDisplayNames()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }
