/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Support.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.elmo.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Locale;
import java.util.SortedSet;
import org.openide.util.Lookup;
import org.openrdf.elmo.Entity;
import it.tidalwave.util.As.NotFoundBehaviour;
import it.tidalwave.openrdf.elmo.ElmoAsSupport;
import it.tidalwave.openrdf.elmo.ElmoUtils;
import it.tidalwave.openrdf.elmo.ElmoVolatile;
import it.tidalwave.openrdf.elmo.RDFUtils;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: Support.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 ******************************************************************************/
public class Support
  {
    @CheckForNull
    @ElmoVolatile
    private Lookup lookup;

    @Nonnull
    protected final Entity entity;

//    public Support()
//      {
//        entity = null;
//      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected Support (@Nonnull final Entity entity)
      {
        this.entity = entity;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public <T> T as (@Nonnull final Class<T> clazz)
      {
        return ElmoAsSupport.as(entity, clazz); // FIXME
//        return getLookup().lookup(As.class);.as(clazz);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public <T> T as (@Nonnull final Class<T> clazz, @Nonnull final NotFoundBehaviour<T> notFoundBehaviour)
      {
        return ElmoAsSupport.as(entity, clazz, notFoundBehaviour); // FIXME
//        return getLookup().lookup(As.class).as(clazz, notFoundBehaviour);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public String getDisplayName()
      {
        return RDFUtils.getDisplayName(entity);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public String getDisplayName (@Nonnull final Locale locale)
      {
        return RDFUtils.getDisplayName(entity, locale);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public SortedSet<Locale> getLocales()
      {
        return RDFUtils.getLocales(entity);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public synchronized Lookup getLookup()
      {
        if (lookup == null)
          {
            lookup = ElmoUtils.createLookup(entity, this);
          }

        return lookup;
      }
  }
